@@ -5,12 +5,21 @@ from subprocess import CompletedProcess
 import pytest
 
 from hatch.config.constants import ConfigEnvVars
-from hatch.env.internal.fmt import PER_FILE_IGNORED_RULES, PREVIEW_RULES, STABLE_RULES
 from hatch.project.core import Project
 
 
 def construct_ruff_defaults_file(rules: tuple[str, ...]) -> str:
-    lines = ['line-length = 120', '', '[lint]']
+    from hatch.cli.fmt.core import PER_FILE_IGNORED_RULES
+
+    lines = [
+        'line-length = 120',
+        '',
+        '[format]',
+        'docstring-code-format = true',
+        'docstring-code-line-length = 80',
+        '',
+        '[lint]',
+    ]
 
     # Selected rules
     lines.append('select = [')
@@ -46,11 +55,15 @@ def construct_ruff_defaults_file(rules: tuple[str, ...]) -> str:
 
 @pytest.fixture(scope='module')
 def defaults_file_stable() -> str:
+    from hatch.cli.fmt.core import STABLE_RULES
+
     return construct_ruff_defaults_file(STABLE_RULES)
 
 
 @pytest.fixture(scope='module')
 def defaults_file_preview() -> str:
+    from hatch.cli.fmt.core import PREVIEW_RULES, STABLE_RULES
+
     return construct_ruff_defaults_file(STABLE_RULES + PREVIEW_RULES)
 
 
@@ -78,9 +91,9 @@ class TestDefaults:
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt')
@@ -88,7 +101,7 @@ class TestDefaults:
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -132,9 +145,9 @@ extend = "{config_path}\""""
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--check')
@@ -142,7 +155,7 @@ extend = "{config_path}\""""
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -170,6 +183,63 @@ extend = "{config_path}\""""
 extend = "{config_path}\""""
         )
 
+    def test_existing_config(self, hatch, temp_dir, config_file, mocker, ruff_on_path, defaults_file_stable):
+        config_file.model.template.plugins['default']['tests'] = False
+        config_file.save()
+
+        project_name = 'My.App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+        data_path = temp_dir / 'data'
+        data_path.mkdir()
+
+        project_file = project_path / 'pyproject.toml'
+        old_contents = project_file.read_text()
+        project_file.write_text(f'[tool.ruff]\n{old_contents}')
+
+        run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
+
+        with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
+            result = hatch('fmt', '--check')
+
+        assert result.exit_code == 0, result.output
+        assert not result.output
+
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
+        config_dir = next(root_data_path.iterdir())
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+
+        assert run.call_args_list == [
+            mocker.call(
+                [ruff_on_path, 'check', '--config', str(user_config), '.'],
+                shell=False,
+            ),
+            mocker.call(
+                [ruff_on_path, 'format', '--config', str(user_config), '--check', '--diff', '.'],
+                shell=False,
+            ),
+        ]
+
+        assert default_config.read_text() == defaults_file_stable
+
+        config_path = str(default_config).replace('\\', '\\\\')
+        assert (
+            user_config.read_text()
+            == f"""\
+[tool.ruff]
+extend = "{config_path}\"
+{old_contents.rstrip()}"""
+        )
+
 
 class TestPreview:
     def test_fix_flag(self, hatch, temp_dir, config_file, mocker, ruff_on_path, defaults_file_preview):
@@ -188,9 +258,9 @@ class TestPreview:
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--preview')
@@ -198,7 +268,7 @@ class TestPreview:
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -242,9 +312,9 @@ extend = "{config_path}\""""
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--check', '--preview')
@@ -252,7 +322,7 @@ extend = "{config_path}\""""
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -299,9 +369,9 @@ extend = "{config_path}\""""
         original_user_config.write_text(f'{original_user_config.read_text()}\n[tool.ruff.lint]\npreview = true')
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--check')
@@ -309,7 +379,7 @@ extend = "{config_path}\""""
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -333,6 +403,7 @@ extend = "{config_path}\""""
             user_config.read_text()
             == f"""\
 {old_contents}
+
 [tool.ruff]
 extend = "{config_path}\""""
         )
@@ -356,9 +427,9 @@ extend = "{config_path}\""""
         original_user_config.write_text(f'{original_user_config.read_text()}\n[tool.ruff.format]\npreview = true')
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--check')
@@ -366,7 +437,7 @@ extend = "{config_path}\""""
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -390,6 +461,7 @@ extend = "{config_path}\""""
             user_config.read_text()
             == f"""\
 {old_contents}
+
 [tool.ruff]
 extend = "{config_path}\""""
         )
@@ -412,9 +484,9 @@ class TestComponents:
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--linter')
@@ -422,7 +494,7 @@ class TestComponents:
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -462,9 +534,9 @@ extend = "{config_path}\""""
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--formatter')
@@ -472,7 +544,7 @@ extend = "{config_path}\""""
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -512,9 +584,9 @@ extend = "{config_path}\""""
         data_path.mkdir()
 
         mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--linter', '--formatter')
@@ -544,9 +616,9 @@ class TestArguments:
         data_path.mkdir()
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--', '--foo', 'bar')
@@ -554,7 +626,7 @@ class TestArguments:
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         config_dir = next(root_data_path.iterdir())
         default_config = config_dir / 'ruff_defaults.toml'
         user_config = config_dir / 'pyproject.toml'
@@ -600,9 +672,9 @@ class TestConfigPath:
         data_path.mkdir()
 
         mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--sync')
@@ -633,14 +705,14 @@ class TestConfigPath:
 
         project = Project(project_path)
         config = dict(project.raw_config)
-        config['tool']['hatch']['format'] = {'config-path': 'ruff_defaults.toml'}
+        config['tool']['hatch']['envs'] = {'hatch-static-analysis': {'config-path': 'ruff_defaults.toml'}}
         config['tool']['ruff'] = {'extend': 'ruff_defaults.toml'}
         project.save_config(config)
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--sync')
@@ -648,7 +720,7 @@ class TestConfigPath:
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
 
         assert run.call_args_list == [
@@ -683,14 +755,14 @@ class TestConfigPath:
 
         project = Project(project_path)
         config = dict(project.raw_config)
-        config['tool']['hatch']['format'] = {'config-path': 'ruff_defaults.toml'}
+        config['tool']['hatch']['envs'] = {'hatch-static-analysis': {'config-path': 'ruff_defaults.toml'}}
         config['tool']['ruff'] = {'extend': 'ruff_defaults.toml'}
         project.save_config(config)
 
         run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.exists', return_value=True)
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.dependency_hash', return_value='')
-        mocker.patch('hatch.env.internal.fmt.InternalFormatEnvironment.command_context')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
 
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt')
@@ -698,7 +770,7 @@ class TestConfigPath:
         assert result.exit_code == 0, result.output
         assert not result.output
 
-        root_data_path = data_path / 'env' / '.internal' / 'fmt' / '.config'
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
 
         assert run.call_args_list == [
@@ -713,3 +785,56 @@ class TestConfigPath:
         ]
 
         assert not default_config_file.read_text()
+
+    def test_sync_legacy_config(self, hatch, temp_dir, config_file, mocker, ruff_on_path, defaults_file_stable):
+        config_file.model.template.plugins['default']['tests'] = False
+        config_file.save()
+
+        project_name = 'My.App'
+
+        with temp_dir.as_cwd():
+            result = hatch('new', project_name)
+
+        assert result.exit_code == 0, result.output
+
+        project_path = temp_dir / 'my-app'
+        data_path = temp_dir / 'data'
+        data_path.mkdir()
+        default_config_file = project_path / 'ruff_defaults.toml'
+        assert not default_config_file.is_file()
+
+        project = Project(project_path)
+        config = dict(project.raw_config)
+        config['tool']['hatch']['format'] = {'config-path': 'ruff_defaults.toml'}
+        config['tool']['ruff'] = {'extend': 'ruff_defaults.toml'}
+        project.save_config(config)
+
+        run = mocker.patch('subprocess.run', return_value=CompletedProcess([], 0, stdout=b''))
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.exists', return_value=True)
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.dependency_hash', return_value='')
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.command_context')
+
+        with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
+            result = hatch('fmt', '--sync')
+
+        assert result.exit_code == 0, result.output
+        assert result.output == (
+            'The `tool.hatch.format.config-path` option is deprecated and will be removed in a future release. '
+            'Use `tool.hatch.envs.hatch-static-analysis.config-path` instead.\n'
+        )
+
+        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
+        assert not root_data_path.is_dir()
+
+        assert run.call_args_list == [
+            mocker.call(
+                [ruff_on_path, 'check', '--fix', '.'],
+                shell=False,
+            ),
+            mocker.call(
+                [ruff_on_path, 'format', '.'],
+                shell=False,
+            ),
+        ]
+
+        assert default_config_file.read_text() == defaults_file_stable
