@@ -13,7 +13,7 @@ def get_files(**kwargs):
             ),
         ),
         File(
-            Path(kwargs['package_name'], '__init__.py'),
+            Path('src', kwargs['package_name'], '__init__.py'),
             f"""\
 # SPDX-FileCopyrightText: {kwargs['year']}-present {kwargs['author']} <{kwargs['email']}>
 #
@@ -21,44 +21,43 @@ def get_files(**kwargs):
 """,
         ),
         File(
-            Path(kwargs['package_name'], '__about__.py'),
+            Path('src', kwargs['package_name'], '__about__.py'),
             f"""\
 # SPDX-FileCopyrightText: {kwargs['year']}-present {kwargs['author']} <{kwargs['email']}>
 #
 # SPDX-License-Identifier: MIT
-__version__ = '0.0.1'
+__version__ = "0.0.1"
 """,
         ),
         File(
-            Path(kwargs['package_name'], '__main__.py'),
+            Path('src', kwargs['package_name'], '__main__.py'),
             f"""\
 # SPDX-FileCopyrightText: {kwargs['year']}-present {kwargs['author']} <{kwargs['email']}>
 #
 # SPDX-License-Identifier: MIT
 import sys
 
-if __name__ == '__main__':
-    from .cli import {kwargs['package_name']}
+if __name__ == "__main__":
+    from {kwargs['package_name']}.cli import {kwargs['package_name']}
 
     sys.exit({kwargs['package_name']}())
 """,
         ),
         File(
-            Path(kwargs['package_name'], 'cli', '__init__.py'),
+            Path('src', kwargs['package_name'], 'cli', '__init__.py'),
             f"""\
 # SPDX-FileCopyrightText: {kwargs['year']}-present {kwargs['author']} <{kwargs['email']}>
 #
 # SPDX-License-Identifier: MIT
 import click
 
-from ..__about__ import __version__
+from {kwargs['package_name']}.__about__ import __version__
 
 
-@click.group(context_settings={{'help_option_names': ['-h', '--help']}}, invoke_without_command=True)
-@click.version_option(version=__version__, prog_name={kwargs['project_name']!r})
-@click.pass_context
-def {kwargs['package_name']}(ctx: click.Context):
-    click.echo('Hello world!')
+@click.group(context_settings={{"help_option_names": ["-h", "--help"]}}, invoke_without_command=True)
+@click.version_option(version=__version__, prog_name="{kwargs['project_name']}")
+def {kwargs['package_name']}():
+    click.echo("Hello world!")
 """,
         ),
         File(
@@ -104,9 +103,10 @@ build-backend = "hatchling.build"
 
 [project]
 name = "{kwargs['project_name_normalized']}"
+dynamic = ["version"]
 description = ''
 readme = "README.md"
-requires-python = ">=3.7"
+requires-python = ">=3.8"
 license = "MIT"
 keywords = []
 authors = [
@@ -115,18 +115,17 @@ authors = [
 classifiers = [
   "Development Status :: 4 - Beta",
   "Programming Language :: Python",
-  "Programming Language :: Python :: 3.7",
   "Programming Language :: Python :: 3.8",
   "Programming Language :: Python :: 3.9",
   "Programming Language :: Python :: 3.10",
   "Programming Language :: Python :: 3.11",
+  "Programming Language :: Python :: 3.12",
   "Programming Language :: Python :: Implementation :: CPython",
   "Programming Language :: Python :: Implementation :: PyPy",
 ]
 dependencies = [
   "click",
 ]
-dynamic = ["version"]
 
 [project.urls]
 Documentation = "https://github.com/unknown/{kwargs['project_name_normalized']}#readme"
@@ -137,33 +136,53 @@ Source = "https://github.com/unknown/{kwargs['project_name_normalized']}"
 {kwargs['project_name_normalized']} = "{kwargs['package_name']}.cli:{kwargs['package_name']}"
 
 [tool.hatch.version]
-path = "{kwargs['package_name']}/__about__.py"
+path = "src/{kwargs['package_name']}/__about__.py"
 
 [tool.hatch.envs.default]
 dependencies = [
+  "coverage[toml]>=6.5",
   "pytest",
-  "pytest-cov",
 ]
 [tool.hatch.envs.default.scripts]
-cov = "pytest --cov-report=term-missing --cov-config=pyproject.toml --cov={kwargs['package_name']} --cov=tests {{args}}"
-no-cov = "cov --no-cov {{args}}"
+test = "pytest {{args:tests}}"
+test-cov = "coverage run -m pytest {{args:tests}}"
+cov-report = [
+  "- coverage combine",
+  "coverage report",
+]
+cov = [
+  "test-cov",
+  "cov-report",
+]
 
-[[tool.hatch.envs.test.matrix]]
-python = ["37", "38", "39", "310", "311"]
+[[tool.hatch.envs.all.matrix]]
+python = ["3.8", "3.9", "3.10", "3.11", "3.12"]
+
+[tool.hatch.envs.types]
+dependencies = [
+  "mypy>=1.0.0",
+]
+[tool.hatch.envs.types.scripts]
+check = "mypy --install-types --non-interactive {{args:src/{kwargs['package_name']} tests}}"
 
 [tool.coverage.run]
+source_pkgs = ["{kwargs['package_name']}", "tests"]
 branch = true
 parallel = true
 omit = [
-  "{kwargs['package_name']}/__about__.py",
+  "src/{kwargs['package_name']}/__about__.py",
 ]
 
+[tool.coverage.paths]
+{kwargs['package_name']} = ["src/{kwargs['package_name']}", "*/{kwargs['project_name_normalized']}/src/{kwargs['package_name']}"]
+tests = ["tests", "*/{kwargs['project_name_normalized']}/tests"]
+
 [tool.coverage.report]
 exclude_lines = [
   "no cov",
   "if __name__ == .__main__.:",
   "if TYPE_CHECKING:",
 ]
-""",
+""",  # noqa: E501
         ),
-    ]  # noqa: E501
+    ]
